#!/bin/bash
# Script de Demarrage TP UML 
# Adaptation des scripts de M. Billaud 
# Modofies par A. Guermouche, A. Esnard 
# Universite de Bordeaux 1

export PATH=$PATH:.

### Parameters 

UML_DIR=`dirname $0`        # directory of this script
UML_UTILITIES_DIR="/usr"    # standard install of uml-utilities package
LINUX=${UML_DIR}/linux
ROOT_FS_ARCHIVE=${UML_DIR}/root_fs.tgz
UML_COW_TMP=/tmp/UML-COW-TMP
ROOT_FS_DST_DIR=/tmp        # target dir on my Laptop
# ROOT_FS_DST_DIR=/espace   # target dir on CREMI
UML_MEM="150M"              # 80M-200M

### Usage

if [ $# -eq 2 ] 
then 
    TP_CONFIG=$1
    TP_ARCHIVE=$2 
elif [ $# -eq 5 ] 
then 
    TP_CONFIG=$1
    TP_ARCHIVE=$2 
    ROOT_FS_ARCHIVE=$3
    LINUX=$4
    UML_COW_TMP=$5
else
    echo "Usage: $0 TP_CONFIG TP_ARCHIVE [ROOT_FS_ARCHIVE LINUX UML_COW_TMP]"
    exit
fi     

ROOT_FS=${ROOT_FS_DST_DIR}/`basename ${ROOT_FS_ARCHIVE} .tgz`
# ROOT_FS=${ROOT_FS_DST_DIR}/root_fs
UML_TMP=`mktemp -d /tmp/UML-TMP-XXXXXX`

### check

echo "TP_CONFIG: ${TP_CONFIG}"
echo "TP_ARCHIVE: ${TP_ARCHIVE}"
echo "UML_DIR: ${UML_DIR}"
echo "UML_UTILITIES_DIR: ${UML_UTILITIES_DIR}"
echo "LINUX: ${LINUX}"
echo "UML_COW_TMP: ${UML_COW_TMP}"
echo "ROOT_FS_ARCHIVE: ${ROOT_FS_ARCHIVE}"
echo "ROOT_FS_DST_DIR: ${ROOT_FS_DST_DIR}"
echo "ROOT_FS: ${ROOT_FS}"

### Init...

if ! [ -d "${UML_COW_TMP}" ]
then
    mkdir -p ${UML_COW_TMP}
fi

### Check 

if ! [ -e "/usr/bin/dirname" ]
then
   echo "You need coreutils package: /usr/bin/dirname ."
   exit  
elif ! [ -e "/usr/bin/basename" ]
then
   echo "You need coreutils package: /usr/bin/basename."
   exit  
elif ! [ -e "/usr/lib/uml/port-helper" ]
then
   echo "You need to install the package uml-utilities (required by UML)."
   exit  
elif ! [ -e "/usr/bin/uml_switch" ]
then
   echo "You need to install the package uml-utilities (required by UML)."
   exit  
elif ! [ -d "${UML_DIR}" ]
then
    echo "Bad UML Directory: ${UML_DIR}"
    exit
elif ! [ -d "${ROOT_FS_DST_DIR}" ]
then
    echo "Bad Root File System Directory: ${ROOT_FS_DST_DIR}"
    exit
elif ! [ -d "${UML_COW_TMP}" ]
then
    echo "Bad UML COW Temporary Directory: ${UML_COW_TMP}"
    exit
elif ! [ -e "${TP_ARCHIVE}" ]
then
    echo "Bad Archive File: ${TP_ARCHIVE}"
    exit
elif ! [ -e "${TP_CONFIG}" ]
then
    echo "Bad Config File: ${TP_CONFIG}"
    exit
elif ! [ -x "${LINUX}" ]
then
    echo "UML Linux Kernel Not Found or Not Executable: ${LINUX}"
    exit
fi

### Install UML Root File System

if ! [ -e "${ROOT_FS}" ]
then
  echo "Decompressing ${ROOT_FS_ARCHIVE} in ${ROOT_FS_DST_DIR}"
  tar xvzf ${ROOT_FS_ARCHIVE} -C ${ROOT_FS_DST_DIR}
  # check
  if ! [ -e "${ROOT_FS}" ] ; then
      echo "Error! Root File System ${ROOT_FS} not found!"
      echo "Check this file and re-launch the script."
      exit
  fi
  chmod 777 ${ROOT_FS}
fi

### UML Temp (UML mount point = /mnt/host)

rm -rf ${HOME}/UML-TMP
ln -sf ${UML_TMP} ${HOME}/UML-TMP

### UML COW TMP

echo "Decompressing ${TP_ARCHIVE} in ${UML_COW_TMP}"
tar xvzf ${TP_ARCHIVE} -C ${UML_COW_TMP}

### UML Functions

export MY_HOST=1
export SWITCH_LIST=""

SWITCH () {
        name=$1
        ALL_SWITCHES="$ALL_SWITCHES $name"
        s=`mktemp /tmp/$name.ctl.XXXXXX`
	SWITCH_LIST="$SWITCH_LIST $s "
        fuser -k $s
        ${UML_UTILITIES_DIR}/bin/uml_switch -unix $s &
        SWITCHPIDS="$SWITCHPIDS $!"
}

STOP_ALL_SWITCHES () {
        kill -9 $SWITCHPIDS
	rm -f $SWITCH_LIST
}

HOST () {
        name=$1
        mkdir ${UML_TMP}/$name
        chmod -R a+rwx ${UML_TMP}/$name
	les_machines="$les_machines $name" 
        cmd="$LINUX  mem=${UML_MEM} \
                     con1=xterm \
                     con=null \
                     xterm=gnome-terminal,-t,-x \
                     hostfs=${UML_TMP}/$name \
                     umid=$name \
                     devfs=nomount \
                     ubd0=${UML_COW_TMP}/${name}.cow,$ROOT_FS \
                     my_umid=$name" 
        i=0
        shift 1
        for sw in $* ; do
	        t=`echo /tmp/ | sed 's/\//\\\\\//g'`
		s=`echo $SWITCH_LIST | sed "s/.*\(${t}${sw}\.ctl\.......\).*/\1/"`
                cmd="$cmd eth$i=daemon,a2:00:00:00:${MY_HOST}:${i},,$s" 
                i=`expr ${i} + 1`
        done
        let MY_HOST=MY_HOST+1
        echo $cmd
        (
                 $cmd
        ) &
        hpid="$hpid $!" 
} 

WAIT_FOR_HOSTS () {
        for p in $hpid ; do wait $p ; done
}

RUN () {
    WAIT_FOR_HOSTS
    STOP_ALL_SWITCHES
}

END () {

    rm -rf  ${UML_TMP}

    echo
    echo
    echo "If you have properly halted UMLs, you can save your works" 
    echo "with this command:"
    echo
    echo "  cd ${UML_COW_TMP} ; tar cvzSf  myarchive.tgz *.cow ; cd -"
    echo

    exit
}

### trap

trap END INT # TERM EXIT

### load config and run it

source ${TP_CONFIG}

RUN

### end

END()

